@@ -7,12 +7,14 @@ use cache::CredentialsCache;
 pub use credentials::Credentials;
 pub use keyring::KeyringProvider;
 pub use middleware::AuthMiddleware;
+pub use policy::{AuthPolicy, UrlAuthPolicies};
 use realm::Realm;
 
 mod cache;
 mod credentials;
 mod keyring;
 mod middleware;
+mod policy;
 mod realm;
 
 // TODO(zanieb): Consider passing a cache explicitly throughout
