@@ -1,4 +1,5 @@
 mod metadata;
+mod serde_verbatim;
 mod source_dist;
 mod wheel;
 
@@ -14,6 +15,7 @@ use thiserror::Error;
 use tracing::debug;
 use uv_fs::Simplified;
 use uv_globfilter::PortableGlobError;
+use uv_pypi_types::IdentifierParseError;
 
 #[derive(Debug, Error)]
 pub enum Error {
@@ -23,6 +25,8 @@ pub enum Error {
     Toml(#[from] toml::de::Error),
     #[error("Invalid pyproject.toml")]
     Validation(#[from] ValidationError),
+    #[error(transparent)]
+    Identifier(#[from] IdentifierParseError),
     #[error("Unsupported glob expression in: `{field}`")]
     PortableGlob {
         field: String,
@@ -394,8 +398,8 @@ mod tests {
             license = { file = "license.txt" }
 
             [build-system]
-            requires = ["uv>=0.5.15,<0.6"]
-            build-backend = "uv"
+            requires = ["uv_build>=0.5.15,<0.6"]
+            build-backend = "uv_build"
         "#
             },
         )
@@ -462,8 +466,8 @@ mod tests {
             version = "1.0.0"
 
             [build-system]
-            requires = ["uv>=0.5.15,<0.6"]
-            build-backend = "uv"
+            requires = ["uv_build>=0.5.15,<0.6"]
+            build-backend = "uv_build"
         "#
             },
         )
