@@ -1,4 +1,3 @@
-use std::cmp::max;
 use std::fmt::{Display, Formatter};
 use std::io;
 use std::io::Write;
@@ -13,7 +12,6 @@ use tracing::debug;
 pub use archive::ArchiveId;
 use uv_cache_info::Timestamp;
 use uv_distribution_filename::WheelFilename;
-use uv_distribution_types::InstalledDist;
 use uv_fs::{cachedir, directories, LockedFile};
 use uv_normalize::PackageName;
 use uv_pypi_types::ResolutionMetadata;
@@ -360,43 +358,7 @@ impl Cache {
     /// Returns the number of entries removed from the cache.
     pub fn remove(&self, name: &PackageName) -> Result<Removal, io::Error> {
         // Collect the set of referenced archives.
-        let before = {
-            let mut references = FxHashSet::default();
-            for bucket in CacheBucket::iter() {
-                let bucket = self.bucket(bucket);
-                if bucket.is_dir() {
-                    for entry in walkdir::WalkDir::new(bucket) {
-                        let entry = entry?;
-
-                        // Ignore any `.lock` files.
-                        if entry
-                            .path()
-                            .extension()
-                            .is_some_and(|ext| ext.eq_ignore_ascii_case("lock"))
-                        {
-                            continue;
-                        }
-
-                        // Identify entries that match the wheel stem pattern (e.g., `typing-extensions-4.8.0-py3-none-any`).
-                        let Some(filename) = entry
-                            .path()
-                            .file_name()
-                            .and_then(|file_name| file_name.to_str())
-                        else {
-                            continue;
-                        };
-
-                        if WheelFilename::from_stem(filename).is_err() {
-                            continue;
-                        }
-                        if let Ok(target) = self.resolve_link(entry.path()) {
-                            references.insert(target);
-                        }
-                    }
-                }
-            }
-            references
-        };
+        let before = self.find_archive_references()?;
 
         // Remove any entries for the package from the cache.
         let mut summary = Removal::default();
@@ -405,42 +367,7 @@ impl Cache {
         }
 
         // Collect the set of referenced archives after the removal.
-        let after = {
-            let mut references = FxHashSet::default();
-            for bucket in CacheBucket::iter() {
-                let bucket = self.bucket(bucket);
-                if bucket.is_dir() {
-                    for entry in walkdir::WalkDir::new(bucket) {
-                        let entry = entry?;
-
-                        // Ignore any `.lock` files.
-                        if entry
-                            .path()
-                            .extension()
-                            .is_some_and(|ext| ext.eq_ignore_ascii_case("lock"))
-                        {
-                            continue;
-                        }
-
-                        // Identify entries that match the wheel stem pattern (e.g., `typing-extensions-4.8.0-py3-none-any`).
-                        let Some(filename) = entry
-                            .path()
-                            .file_name()
-                            .and_then(|file_name| file_name.to_str())
-                        else {
-                            continue;
-                        };
-                        if WheelFilename::from_stem(filename).is_err() {
-                            continue;
-                        }
-                        if let Ok(target) = self.resolve_link(entry.path()) {
-                            references.insert(target);
-                        }
-                    }
-                }
-            }
-            references
-        };
+        let after = self.find_archive_references()?;
 
         if before != after {
             // Remove any archives that are no longer referenced.
@@ -562,40 +489,7 @@ impl Cache {
         }
 
         // Fourth, remove any unused archives (by searching for archives that are not symlinked).
-        let mut references = FxHashSet::default();
-
-        for bucket in CacheBucket::iter() {
-            let bucket = self.bucket(bucket);
-            if bucket.is_dir() {
-                for entry in walkdir::WalkDir::new(bucket) {
-                    let entry = entry?;
-
-                    // Ignore any `.lock` files.
-                    if entry
-                        .path()
-                        .extension()
-                        .is_some_and(|ext| ext.eq_ignore_ascii_case("lock"))
-                    {
-                        continue;
-                    }
-
-                    // Identify entries that match the wheel stem pattern (e.g., `typing-extensions-4.8.0-py3-none-any`).
-                    let Some(filename) = entry
-                        .path()
-                        .file_name()
-                        .and_then(|file_name| file_name.to_str())
-                    else {
-                        continue;
-                    };
-                    if WheelFilename::from_stem(filename).is_err() {
-                        continue;
-                    }
-                    if let Ok(target) = self.resolve_link(entry.path()) {
-                        references.insert(target);
-                    }
-                }
-            }
-        }
+        let references = self.find_archive_references()?;
 
         match fs_err::read_dir(self.bucket(CacheBucket::Archive)) {
             Ok(entries) => {
@@ -615,6 +509,63 @@ impl Cache {
         Ok(summary)
     }
 
+    /// Find all references to entries in the archive bucket.
+    ///
+    /// Archive entries are often referenced by symlinks in other cache buckets. This method
+    /// searches for all such references.
+    fn find_archive_references(&self) -> Result<FxHashSet<PathBuf>, io::Error> {
+        let mut references = FxHashSet::default();
+        for bucket in CacheBucket::iter() {
+            let bucket_path = self.bucket(bucket);
+            if bucket_path.is_dir() {
+                for entry in walkdir::WalkDir::new(bucket_path) {
+                    let entry = entry?;
+
+                    // Ignore any `.lock` files.
+                    if entry
+                        .path()
+                        .extension()
+                        .is_some_and(|ext| ext.eq_ignore_ascii_case("lock"))
+                    {
+                        continue;
+                    }
+
+                    let Some(filename) = entry
+                        .path()
+                        .file_name()
+                        .and_then(|file_name| file_name.to_str())
+                    else {
+                        continue;
+                    };
+
+                    if bucket == CacheBucket::Wheels {
+                        // In the `wheels` bucket, we often use a hash of the filename as the
+                        // directory name, so we can't rely on the stem.
+                        //
+                        // Instead, we skip if it contains an extension (e.g., `.whl`, `.http`,
+                        // `.rev`, and `.msgpack` files).
+                        if filename
+                            .rsplit_once('-') // strip version/tags, might contain a dot ('.')
+                            .is_none_or(|(_, suffix)| suffix.contains('.'))
+                        {
+                            continue;
+                        }
+                    } else {
+                        // For other buckets only include entries that match the wheel stem pattern (e.g., `typing-extensions-4.8.0-py3-none-any`).
+                        if WheelFilename::from_stem(filename).is_err() {
+                            continue;
+                        }
+                    }
+
+                    if let Ok(target) = self.resolve_link(entry.path()) {
+                        references.insert(target);
+                    }
+                }
+            }
+        }
+        Ok(references)
+    }
+
     /// Create a link to a directory in the archive bucket.
     ///
     /// On Windows, we write structured data ([`Link`]) to a file containing the archive ID and
@@ -1040,7 +991,7 @@ impl CacheBucket {
             Self::Simple => "simple-v15",
             // Note that when bumping this, you'll also need to bump it
             // in `crates/uv/tests/it/cache_prune.rs`.
-            Self::Wheels => "wheels-v4",
+            Self::Wheels => "wheels-v5",
             // Note that when bumping this, you'll also need to bump
             // `ARCHIVE_VERSION` in `crates/uv-cache/src/lib.rs`.
             Self::Archive => "archive-v0",
@@ -1074,14 +1025,14 @@ impl CacheBucket {
                 // For alternate indices, we expect a directory for every index (under an `index`
                 // subdirectory), followed by a directory per package (indexed by name).
                 let root = cache.bucket(self).join(WheelCacheKind::Index);
-                for directory in directories(root) {
+                for directory in directories(root)? {
                     summary += rm_rf(directory.join(name.to_string()))?;
                 }
 
                 // For direct URLs, we expect a directory for every URL, followed by a
                 // directory per package (indexed by name).
                 let root = cache.bucket(self).join(WheelCacheKind::Url);
-                for directory in directories(root) {
+                for directory in directories(root)? {
                     summary += rm_rf(directory.join(name.to_string()))?;
                 }
             }
@@ -1093,7 +1044,7 @@ impl CacheBucket {
                 // For alternate indices, we expect a directory for every index (under an `index`
                 // subdirectory), followed by a directory per package (indexed by name).
                 let root = cache.bucket(self).join(WheelCacheKind::Index);
-                for directory in directories(root) {
+                for directory in directories(root)? {
                     summary += rm_rf(directory.join(name.to_string()))?;
                 }
 
@@ -1101,8 +1052,8 @@ impl CacheBucket {
                 // directory per version. To determine whether the URL is relevant, we need to
                 // search for a wheel matching the package name.
                 let root = cache.bucket(self).join(WheelCacheKind::Url);
-                for url in directories(root) {
-                    if directories(&url).any(|version| is_match(&version, name)) {
+                for url in directories(root)? {
+                    if directories(&url)?.any(|version| is_match(&version, name)) {
                         summary += rm_rf(url)?;
                     }
                 }
@@ -1111,8 +1062,8 @@ impl CacheBucket {
                 // directory per version. To determine whether the path is relevant, we need to
                 // search for a wheel matching the package name.
                 let root = cache.bucket(self).join(WheelCacheKind::Path);
-                for path in directories(root) {
-                    if directories(&path).any(|version| is_match(&version, name)) {
+                for path in directories(root)? {
+                    if directories(&path)?.any(|version| is_match(&version, name)) {
                         summary += rm_rf(path)?;
                     }
                 }
@@ -1121,8 +1072,8 @@ impl CacheBucket {
                 // directory for every SHA. To determine whether the SHA is relevant, we need to
                 // search for a wheel matching the package name.
                 let root = cache.bucket(self).join(WheelCacheKind::Git);
-                for repository in directories(root) {
-                    for sha in directories(repository) {
+                for repository in directories(root)? {
+                    for sha in directories(repository)? {
                         if is_match(&sha, name) {
                             summary += rm_rf(sha)?;
                         }
@@ -1137,7 +1088,7 @@ impl CacheBucket {
                 // For alternate indices, we expect a directory for every index (under an `index`
                 // subdirectory), followed by a directory per package (indexed by name).
                 let root = cache.bucket(self).join(WheelCacheKind::Index);
-                for directory in directories(root) {
+                for directory in directories(root)? {
                     summary += rm_rf(directory.join(format!("{name}.rkyv")))?;
                 }
             }
@@ -1190,123 +1141,6 @@ impl Display for CacheBucket {
     }
 }
 
-/// A timestamp for an archive, which could be a directory (in which case the modification time is
-/// the latest modification time of the `pyproject.toml`, `setup.py`, or `setup.cfg` file in the
-/// directory) or a single file.
-#[derive(Debug, Clone, Copy, PartialEq, Eq)]
-pub struct ArchiveTimestamp(Timestamp);
-
-impl ArchiveTimestamp {
-    /// Return the modification timestamp for an archive, which could be a file (like a wheel or a zip
-    /// archive) or a directory containing a Python package.
-    ///
-    /// If the path is to a directory with no entrypoint (i.e., no `pyproject.toml`, `setup.py`, or
-    /// `setup.cfg`), returns `None`.
-    pub fn from_path(path: impl AsRef<Path>) -> Result<Option<Self>, io::Error> {
-        let metadata = fs_err::metadata(path.as_ref())?;
-        if metadata.is_file() {
-            Ok(Some(Self(Timestamp::from_metadata(&metadata))))
-        } else {
-            Self::from_source_tree(path)
-        }
-    }
-
-    /// Return the modification timestamp for a file.
-    pub fn from_file(path: impl AsRef<Path>) -> Result<Self, io::Error> {
-        let metadata = fs_err::metadata(path.as_ref())?;
-        Ok(Self(Timestamp::from_metadata(&metadata)))
-    }
-
-    /// Return the modification timestamp for a source tree, i.e., a directory.
-    ///
-    /// If the source tree doesn't contain an entrypoint (i.e., no `pyproject.toml`, `setup.py`, or
-    /// `setup.cfg`), returns `None`.
-    pub fn from_source_tree(path: impl AsRef<Path>) -> Result<Option<Self>, io::Error> {
-        // Compute the modification timestamp for the `pyproject.toml`, `setup.py`, and
-        // `setup.cfg` files, if they exist.
-        let pyproject_toml = path
-            .as_ref()
-            .join("pyproject.toml")
-            .metadata()
-            .ok()
-            .filter(std::fs::Metadata::is_file)
-            .as_ref()
-            .map(Timestamp::from_metadata);
-
-        let setup_py = path
-            .as_ref()
-            .join("setup.py")
-            .metadata()
-            .ok()
-            .filter(std::fs::Metadata::is_file)
-            .as_ref()
-            .map(Timestamp::from_metadata);
-
-        let setup_cfg = path
-            .as_ref()
-            .join("setup.cfg")
-            .metadata()
-            .ok()
-            .filter(std::fs::Metadata::is_file)
-            .as_ref()
-            .map(Timestamp::from_metadata);
-
-        // Take the most recent timestamp of the three files.
-        let Some(timestamp) = max(pyproject_toml, max(setup_py, setup_cfg)) else {
-            return Ok(None);
-        };
-
-        Ok(Some(Self(timestamp)))
-    }
-
-    /// Return the modification timestamp for an archive.
-    pub fn timestamp(&self) -> Timestamp {
-        self.0
-    }
-
-    /// Returns `true` if the `target` (an installed or cached distribution) is up-to-date with the
-    /// source archive (`source`).
-    ///
-    /// The `target` should be an installed package in a virtual environment, or an unzipped
-    /// package in the cache.
-    ///
-    /// The `source` is a source archive, i.e., a path to a built wheel or a Python package directory.
-    pub fn up_to_date_with(source: &Path, target: ArchiveTarget) -> Result<bool, io::Error> {
-        let Some(modified_at) = Self::from_path(source)? else {
-            // If there's no entrypoint, we can't determine the modification time, so we assume that the
-            // target is not up-to-date.
-            return Ok(false);
-        };
-        let created_at = match target {
-            ArchiveTarget::Install(installed) => {
-                Timestamp::from_path(installed.path().join("METADATA"))?
-            }
-            ArchiveTarget::Cache(cache) => Timestamp::from_path(cache)?,
-        };
-        Ok(modified_at.timestamp() <= created_at)
-    }
-}
-
-#[derive(Debug, Clone, Copy)]
-pub enum ArchiveTarget<'a> {
-    /// The target is an installed package in a virtual environment.
-    Install(&'a InstalledDist),
-    /// The target is an unzipped package in the cache.
-    Cache(&'a Path),
-}
-
-impl PartialOrd for ArchiveTimestamp {
-    fn partial_cmp(&self, other: &Self) -> Option<std::cmp::Ordering> {
-        Some(self.timestamp().cmp(&other.timestamp()))
-    }
-}
-
-impl Ord for ArchiveTimestamp {
-    fn cmp(&self, other: &Self) -> std::cmp::Ordering {
-        self.timestamp().cmp(&other.timestamp())
-    }
-}
-
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum Freshness {
     /// The cache entry is fresh according to the [`Refresh`] policy.
