@@ -1,5 +1,3 @@
-use std::fmt;
-use std::fmt::{Display, Formatter};
 use std::str::FromStr;
 use std::sync::LazyLock;
 
@@ -7,7 +5,7 @@ use serde::{Deserialize, Deserializer, Serialize, Serializer};
 
 use uv_small_str::SmallString;
 
-use crate::{validate_and_normalize_owned, validate_and_normalize_ref, InvalidNameError};
+use crate::{validate_and_normalize_ref, InvalidNameError};
 
 /// The normalized name of a dependency group.
 ///
@@ -20,8 +18,11 @@ pub struct GroupName(SmallString);
 
 impl GroupName {
     /// Create a validated, normalized group name.
-    pub fn new(name: String) -> Result<Self, InvalidNameError> {
-        validate_and_normalize_owned(name).map(Self)
+    ///
+    /// At present, this is no more efficient than calling [`GroupName::from_str`].
+    #[allow(clippy::needless_pass_by_value)]
+    pub fn from_owned(name: String) -> Result<Self, InvalidNameError> {
+        validate_and_normalize_ref(&name).map(Self)
     }
 }
 
@@ -38,8 +39,25 @@ impl<'de> Deserialize<'de> for GroupName {
     where
         D: Deserializer<'de>,
     {
-        let s = String::deserialize(deserializer)?;
-        Self::from_str(&s).map_err(serde::de::Error::custom)
+        struct Visitor;
+
+        impl serde::de::Visitor<'_> for Visitor {
+            type Value = GroupName;
+
+            fn expecting(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+                f.write_str("a string")
+            }
+
+            fn visit_str<E: serde::de::Error>(self, v: &str) -> Result<Self::Value, E> {
+                GroupName::from_str(v).map_err(serde::de::Error::custom)
+            }
+
+            fn visit_string<E: serde::de::Error>(self, v: String) -> Result<Self::Value, E> {
+                GroupName::from_owned(v).map_err(serde::de::Error::custom)
+            }
+        }
+
+        deserializer.deserialize_str(Visitor)
     }
 }
 
@@ -52,8 +70,8 @@ impl Serialize for GroupName {
     }
 }
 
-impl Display for GroupName {
-    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+impl std::fmt::Display for GroupName {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         self.0.fmt(f)
     }
 }
@@ -69,4 +87,4 @@ impl AsRef<str> for GroupName {
 /// Internally, we model dependency groups as a generic concept; but externally, we only expose the
 /// `dev-dependencies` group.
 pub static DEV_DEPENDENCIES: LazyLock<GroupName> =
-    LazyLock::new(|| GroupName::new("dev".to_string()).unwrap());
+    LazyLock::new(|| GroupName::from_str("dev").unwrap());
