@@ -13,15 +13,6 @@ mod extra_name;
 mod group_name;
 mod package_name;
 
-/// Validate and normalize an owned package or extra name.
-pub(crate) fn validate_and_normalize_owned(name: String) -> Result<SmallString, InvalidNameError> {
-    if is_normalized(&name)? {
-        Ok(SmallString::from(name))
-    } else {
-        Ok(SmallString::from(normalize(&name)?))
-    }
-}
-
 /// Validate and normalize an unowned package or extra name.
 pub(crate) fn validate_and_normalize_ref(
     name: impl AsRef<str>,
@@ -151,12 +142,6 @@ mod tests {
                 validate_and_normalize_ref(input).unwrap().as_ref(),
                 "friendly-bard"
             );
-            assert_eq!(
-                validate_and_normalize_owned(input.to_string())
-                    .unwrap()
-                    .as_ref(),
-                "friendly-bard"
-            );
         }
     }
 
@@ -186,12 +171,6 @@ mod tests {
         let unchanged = ["friendly-bard", "1okay", "okay2"];
         for input in unchanged {
             assert_eq!(validate_and_normalize_ref(input).unwrap().as_ref(), input);
-            assert_eq!(
-                validate_and_normalize_owned(input.to_string())
-                    .unwrap()
-                    .as_ref(),
-                input
-            );
             assert!(is_normalized(input).unwrap());
         }
     }
@@ -209,7 +188,6 @@ mod tests {
         ];
         for input in failures {
             assert!(validate_and_normalize_ref(input).is_err());
-            assert!(validate_and_normalize_owned(input.to_string()).is_err());
             assert!(is_normalized(input).is_err());
         }
     }
