@@ -11,6 +11,7 @@ use reqwest::header::AUTHORIZATION;
 use reqwest::multipart::Part;
 use reqwest::{Body, Response, StatusCode};
 use reqwest_middleware::RequestBuilder;
+use reqwest_retry::policies::ExponentialBackoff;
 use reqwest_retry::{RetryPolicy, Retryable, RetryableStrategy};
 use rustc_hash::FxHashSet;
 use serde::Deserialize;
@@ -23,21 +24,22 @@ use tokio::io::{AsyncReadExt, BufReader};
 use tokio::sync::Semaphore;
 use tokio_util::io::ReaderStream;
 use tracing::{debug, enabled, trace, warn, Level};
+use trusted_publishing::TrustedPublishingToken;
 use url::Url;
-use uv_client::{BaseClient, OwnedArchive, RegistryClientBuilder, UvRetryableStrategy};
+use uv_cache::{Cache, Refresh};
+use uv_client::{
+    BaseClient, OwnedArchive, RegistryClientBuilder, UvRetryableStrategy, DEFAULT_RETRIES,
+};
 use uv_configuration::{KeyringProviderType, TrustedPublishing};
 use uv_distribution_filename::{DistFilename, SourceDistExtension, SourceDistFilename};
+use uv_distribution_types::{IndexCapabilities, IndexUrl};
+use uv_extract::hash::{HashReader, Hasher};
 use uv_fs::{ProgressReader, Simplified};
 use uv_metadata::read_metadata_async_seek;
 use uv_pypi_types::{HashAlgorithm, HashDigest, Metadata23, MetadataError};
 use uv_static::EnvVars;
 use uv_warnings::{warn_user, warn_user_once};
 
-pub use trusted_publishing::TrustedPublishingToken;
-use uv_cache::{Cache, Refresh};
-use uv_distribution_types::{IndexCapabilities, IndexUrl};
-use uv_extract::hash::{HashReader, Hasher};
-
 #[derive(Error, Debug)]
 pub enum PublishError {
     #[error("The publish path is not a valid glob pattern: `{0}`")]
@@ -117,9 +119,9 @@ pub enum PublishSendError {
 
 pub trait Reporter: Send + Sync + 'static {
     fn on_progress(&self, name: &str, id: usize);
-    fn on_download_start(&self, name: &str, size: Option<u64>) -> usize;
-    fn on_download_progress(&self, id: usize, inc: u64);
-    fn on_download_complete(&self, id: usize);
+    fn on_upload_start(&self, name: &str, size: Option<u64>) -> usize;
+    fn on_upload_progress(&self, id: usize, inc: u64);
+    fn on_upload_complete(&self, id: usize);
 }
 
 /// Context for using a fresh registry client for check URL requests.
@@ -379,7 +381,8 @@ pub async fn upload(
 
     let mut n_past_retries = 0;
     let start_time = SystemTime::now();
-    let retry_policy = client.retry_policy();
+    // N.B. We cannot use the client policy here because it is set to zero retries
+    let retry_policy = ExponentialBackoff::builder().build_with_max_retries(DEFAULT_RETRIES);
     loop {
         let (request, idx) = build_request(
             file,
@@ -400,7 +403,7 @@ pub async fn upload(
             let retry_decision = retry_policy.should_retry(start_time, n_past_retries);
             if let reqwest_retry::RetryDecision::Retry { execute_after } = retry_decision {
                 warn_user!("Transient failure while handling response for {registry}; retrying...");
-                reporter.on_download_complete(idx);
+                reporter.on_upload_complete(idx);
                 let duration = execute_after
                     .duration_since(SystemTime::now())
                     .unwrap_or_else(|_| Duration::default());
@@ -751,15 +754,16 @@ async fn build_request(
     }
 
     let file = File::open(file).await?;
-    let idx = reporter.on_download_start(&filename.to_string(), Some(file.metadata().await?.len()));
+    let file_size = file.metadata().await?.len();
+    let idx = reporter.on_upload_start(&filename.to_string(), Some(file_size));
     let reader = ProgressReader::new(file, move |read| {
-        reporter.on_download_progress(idx, read as u64);
+        reporter.on_upload_progress(idx, read as u64);
     });
     // Stream wrapping puts a static lifetime requirement on the reader (so the request doesn't have
     // a lifetime) -> callback needs to be static -> reporter reference needs to be Arc'd.
     let file_reader = Body::wrap_stream(ReaderStream::new(reader));
     // See [`files_for_publishing`] on `raw_filename`
-    let part = Part::stream(file_reader).file_name(raw_filename.to_string());
+    let part = Part::stream_with_length(file_reader, file_size).file_name(raw_filename.to_string());
     form = form.part("content", part);
 
     let url = if let Some(username) = username {
@@ -877,11 +881,11 @@ mod tests {
 
     impl Reporter for DummyReporter {
         fn on_progress(&self, _name: &str, _id: usize) {}
-        fn on_download_start(&self, _name: &str, _size: Option<u64>) -> usize {
+        fn on_upload_start(&self, _name: &str, _size: Option<u64>) -> usize {
             0
         }
-        fn on_download_progress(&self, _id: usize, _inc: u64) {}
-        fn on_download_complete(&self, _id: usize) {}
+        fn on_upload_progress(&self, _id: usize, _inc: u64) {}
+        fn on_upload_complete(&self, _id: usize) {}
     }
 
     /// Snapshot the data we send for an upload request for a source distribution.
@@ -985,6 +989,7 @@ mod tests {
                 },
                 headers: {
                     "content-type": "multipart/form-data; boundary=[...]",
+                    "content-length": "6803",
                     "accept": "application/json;q=0.9, text/plain;q=0.8, text/html;q=0.7",
                     "authorization": "Basic ZmVycmlzOkYzUlIhUw==",
                 },
@@ -1135,6 +1140,7 @@ mod tests {
                 },
                 headers: {
                     "content-type": "multipart/form-data; boundary=[...]",
+                    "content-length": "19330",
                     "accept": "application/json;q=0.9, text/plain;q=0.8, text/html;q=0.7",
                     "authorization": "Basic ZmVycmlzOkYzUlIhUw==",
                 },
