@@ -40,10 +40,12 @@ fn sorted_simple_json_files<'de, D: Deserializer<'de>>(d: D) -> Result<Vec<File>
 pub struct File {
     // PEP 714-renamed field, followed by PEP 691-compliant field, followed by non-PEP 691-compliant
     // alias used by PyPI.
+    //
+    // TODO(charlie): Use a single value here and move this into the deserializer, to save space.
     pub core_metadata: Option<CoreMetadata>,
     pub dist_info_metadata: Option<CoreMetadata>,
     pub data_dist_info_metadata: Option<CoreMetadata>,
-    pub filename: String,
+    pub filename: SmallString,
     pub hashes: Hashes,
     /// There are a number of invalid specifiers on PyPI, so we first try to parse it into a
     /// [`VersionSpecifiers`] according to spec (PEP 440), then a [`LenientVersionSpecifiers`] with
@@ -53,7 +55,7 @@ pub struct File {
     pub requires_python: Option<Result<VersionSpecifiers, VersionSpecifiersParseError>>,
     pub size: Option<u64>,
     pub upload_time: Option<Timestamp>,
-    pub url: String,
+    pub url: SmallString,
     pub yanked: Option<Box<Yanked>>,
 }
 
@@ -63,13 +65,37 @@ fn deserialize_version_specifiers_lenient<'de, D>(
 where
     D: Deserializer<'de>,
 {
-    let maybe_string: Option<String> = Option::deserialize(deserializer)?;
-    let Some(string) = maybe_string else {
-        return Ok(None);
-    };
-    Ok(Some(
-        LenientVersionSpecifiers::from_str(&string).map(VersionSpecifiers::from),
-    ))
+    struct Visitor;
+
+    impl<'de> serde::de::Visitor<'de> for Visitor {
+        type Value = Option<Result<VersionSpecifiers, VersionSpecifiersParseError>>;
+
+        fn expecting(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+            f.write_str("a string representing a version specifier")
+        }
+
+        fn visit_str<E: serde::de::Error>(self, v: &str) -> Result<Self::Value, E> {
+            Ok(Some(
+                LenientVersionSpecifiers::from_str(v).map(VersionSpecifiers::from),
+            ))
+        }
+
+        fn visit_some<D>(self, deserializer: D) -> Result<Self::Value, D::Error>
+        where
+            D: Deserializer<'de>,
+        {
+            deserializer.deserialize_str(Visitor)
+        }
+
+        fn visit_none<E>(self) -> Result<Self::Value, E>
+        where
+            E: serde::de::Error,
+        {
+            Ok(None)
+        }
+    }
+
+    deserializer.deserialize_option(Visitor)
 }
 
 #[derive(Debug, Clone)]
