@@ -6,7 +6,6 @@ pub use crate::specification::*;
 pub use crate::unnamed::*;
 
 use uv_distribution_types::{Dist, DistErrorKind, GitSourceDist, SourceDist};
-use uv_git_types::GitUrl;
 use uv_pypi_types::{Requirement, RequirementSource};
 
 mod extras;
@@ -58,24 +57,15 @@ pub(crate) fn required_dist(
             *ext,
         )?,
         RequirementSource::Git {
-            repository,
-            reference,
-            precise,
+            git,
             subdirectory,
             url,
-        } => {
-            let git_url = if let Some(precise) = precise {
-                GitUrl::from_commit(repository.clone(), reference.clone(), *precise)
-            } else {
-                GitUrl::from_reference(repository.clone(), reference.clone())
-            };
-            Dist::Source(SourceDist::Git(GitSourceDist {
-                name: requirement.name.clone(),
-                git: Box::new(git_url),
-                subdirectory: subdirectory.clone(),
-                url: url.clone(),
-            }))
-        }
+        } => Dist::Source(SourceDist::Git(GitSourceDist {
+            name: requirement.name.clone(),
+            git: Box::new(git.clone()),
+            subdirectory: subdirectory.clone(),
+            url: url.clone(),
+        })),
         RequirementSource::Path {
             install_path,
             ext,
