@@ -162,6 +162,14 @@ impl EnvVars {
     /// not use pre-built wheels for the given space-delimited list of packages.
     pub const UV_NO_BINARY_PACKAGE: &'static str = "UV_NO_BINARY_PACKAGE";
 
+    /// Equivalent to the `--no-build` command-line argument. If set, uv will not build
+    /// source distributions.
+    pub const UV_NO_BUILD: &'static str = "UV_NO_BUILD";
+
+    /// Equivalent to the `--no-build-package` command line argument. If set, uv will
+    /// not build source distributions for the given space-delimited list of packages.
+    pub const UV_NO_BUILD_PACKAGE: &'static str = "UV_NO_BUILD_PACKAGE";
+
     /// Equivalent to the `--publish-url` command-line argument. The URL of the upload
     /// endpoint of the index to use with `uv publish`.
     pub const UV_PUBLISH_URL: &'static str = "UV_PUBLISH_URL";
@@ -473,6 +481,11 @@ impl EnvVars {
     #[attr_hidden]
     pub const GIT_SSL_NO_VERIFY: &'static str = "GIT_SSL_NO_VERIFY";
 
+    /// Disable interactive git prompts in terminals, e.g., for credentials. Does not disable
+    /// GUI prompts.
+    #[attr_hidden]
+    pub const GIT_TERMINAL_PROMPT: &'static str = "GIT_TERMINAL_PROMPT";
+
     /// Used in tests for better git isolation.
     ///
     /// For example, we run some tests in ~/.local/share/uv/tests.
@@ -548,6 +561,11 @@ impl EnvVars {
     /// for more.
     pub const RUST_LOG: &'static str = "RUST_LOG";
 
+    /// Add additional context and structure to log messages.
+    ///
+    /// If logging is not enabled, e.g., with `RUST_LOG` or `-v`, this has no effect.
+    pub const UV_LOG_CONTEXT: &'static str = "UV_LOG_CONTEXT";
+
     /// Use to set the stack size used by uv.
     ///
     /// The value is in bytes, and the default is typically 2MB (2097152).
