@@ -7,12 +7,14 @@
 //! Then lowers them into a dependency specification.
 
 use std::collections::BTreeMap;
+use std::fmt::Formatter;
 use std::ops::Deref;
 use std::path::{Path, PathBuf};
 use std::str::FromStr;
 
 use glob::Pattern;
 use owo_colors::OwoColorize;
+use rustc_hash::{FxBuildHasher, FxHashSet};
 use serde::{de::IntoDeserializer, de::SeqAccess, Deserialize, Deserializer, Serialize};
 use thiserror::Error;
 use url::Url;
@@ -25,7 +27,8 @@ use uv_normalize::{ExtraName, GroupName, PackageName};
 use uv_pep440::{Version, VersionSpecifiers};
 use uv_pep508::MarkerTree;
 use uv_pypi_types::{
-    Conflicts, RequirementSource, SchemaConflicts, SupportedEnvironments, VerbatimParsedUrl,
+    Conflicts, DependencyGroups, RequirementSource, SchemaConflicts, SupportedEnvironments,
+    VerbatimParsedUrl,
 };
 
 #[derive(Error, Debug)]
@@ -34,9 +37,13 @@ pub enum PyprojectTomlError {
     TomlSyntax(#[from] toml_edit::TomlError),
     #[error(transparent)]
     TomlSchema(#[from] toml_edit::de::Error),
-    #[error("`pyproject.toml` is using the `[project]` table, but the required `project.name` field is not set")]
+    #[error(
+        "`pyproject.toml` is using the `[project]` table, but the required `project.name` field is not set"
+    )]
     MissingName,
-    #[error("`pyproject.toml` is using the `[project]` table, but the required `project.version` field is neither set nor present in the `project.dynamic` list")]
+    #[error(
+        "`pyproject.toml` is using the `[project]` table, but the required `project.version` field is neither set nor present in the `project.dynamic` list"
+    )]
     MissingVersion,
 }
 
@@ -137,73 +144,6 @@ impl AsRef<[u8]> for PyProjectToml {
     }
 }
 
-/// A specifier item in a [PEP 735](https://peps.python.org/pep-0735/) Dependency Group.
-#[derive(Debug, Clone, PartialEq, Eq)]
-#[cfg_attr(test, derive(Serialize))]
-pub enum DependencyGroupSpecifier {
-    /// A PEP 508-compatible requirement string.
-    Requirement(String),
-    /// A reference to another dependency group.
-    IncludeGroup {
-        /// The name of the group to include.
-        include_group: GroupName,
-    },
-    /// A Dependency Object Specifier.
-    Object(BTreeMap<String, String>),
-}
-
-impl<'de> Deserialize<'de> for DependencyGroupSpecifier {
-    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
-    where
-        D: Deserializer<'de>,
-    {
-        struct Visitor;
-
-        impl<'de> serde::de::Visitor<'de> for Visitor {
-            type Value = DependencyGroupSpecifier;
-
-            fn expecting(&self, formatter: &mut std::fmt::Formatter) -> std::fmt::Result {
-                formatter.write_str("a string or a map with the `include-group` key")
-            }
-
-            fn visit_str<E>(self, value: &str) -> Result<Self::Value, E>
-            where
-                E: serde::de::Error,
-            {
-                Ok(DependencyGroupSpecifier::Requirement(value.to_owned()))
-            }
-
-            fn visit_map<M>(self, mut map: M) -> Result<Self::Value, M::Error>
-            where
-                M: serde::de::MapAccess<'de>,
-            {
-                let mut map_data = BTreeMap::new();
-                while let Some((key, value)) = map.next_entry()? {
-                    map_data.insert(key, value);
-                }
-
-                if map_data.is_empty() {
-                    return Err(serde::de::Error::custom("missing field `include-group`"));
-                }
-
-                if let Some(include_group) = map_data
-                    .get("include-group")
-                    .map(String::as_str)
-                    .map(GroupName::from_str)
-                    .transpose()
-                    .map_err(serde::de::Error::custom)?
-                {
-                    Ok(DependencyGroupSpecifier::IncludeGroup { include_group })
-                } else {
-                    Ok(DependencyGroupSpecifier::Object(map_data))
-                }
-            }
-        }
-
-        deserializer.deserialize_any(Visitor)
-    }
-}
-
 /// PEP 621 project metadata (`project`).
 ///
 /// See <https://packaging.python.org/en/latest/specifications/pyproject-toml>.
@@ -279,6 +219,30 @@ pub struct Tool {
     pub uv: Option<ToolUv>,
 }
 
+/// Validates that index names in the `tool.uv.index` field are unique.
+///
+/// This custom deserializer function checks for duplicate index names
+/// and returns an error if any duplicates are found.
+fn deserialize_index_vec<'de, D>(deserializer: D) -> Result<Option<Vec<Index>>, D::Error>
+where
+    D: Deserializer<'de>,
+{
+    let indexes = Option::<Vec<Index>>::deserialize(deserializer)?;
+    if let Some(indexes) = indexes.as_ref() {
+        let mut seen_names = FxHashSet::with_capacity_and_hasher(indexes.len(), FxBuildHasher);
+        for index in indexes {
+            if let Some(name) = index.name.as_ref() {
+                if !seen_names.insert(name) {
+                    return Err(serde::de::Error::custom(format!(
+                        "duplicate index name `{name}`"
+                    )));
+                }
+            }
+        }
+    }
+    Ok(indexes)
+}
+
 // NOTE(charlie): When adding fields to this struct, mark them as ignored on `Options` in
 // `crates/uv-settings/src/settings.rs`.
 #[derive(Deserialize, OptionsMetadata, Debug, Clone, PartialEq, Eq)]
@@ -341,6 +305,7 @@ pub struct ToolUv {
             url = "https://download.pytorch.org/whl/cu121"
         "#
     )]
+    #[serde(deserialize_with = "deserialize_index_vec", default)]
     pub index: Option<Vec<Index>>,
 
     /// The workspace definition for the project, if any.
@@ -713,8 +678,39 @@ pub struct ToolUvWorkspace {
 }
 
 /// (De)serialize globs as strings.
-#[derive(Serialize, Deserialize, Debug, Clone, PartialEq, Eq)]
-pub struct SerdePattern(#[serde(with = "serde_from_and_to_string")] pub Pattern);
+#[derive(Debug, Clone, PartialEq, Eq)]
+pub struct SerdePattern(Pattern);
+
+impl serde::ser::Serialize for SerdePattern {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: serde::ser::Serializer,
+    {
+        self.0.as_str().serialize(serializer)
+    }
+}
+
+impl<'de> serde::Deserialize<'de> for SerdePattern {
+    fn deserialize<D: serde::Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
+        struct Visitor;
+
+        impl serde::de::Visitor<'_> for Visitor {
+            type Value = SerdePattern;
+
+            fn expecting(&self, f: &mut Formatter) -> std::fmt::Result {
+                f.write_str("a string")
+            }
+
+            fn visit_str<E: serde::de::Error>(self, v: &str) -> Result<Self::Value, E> {
+                Pattern::from_str(v)
+                    .map(SerdePattern)
+                    .map_err(serde::de::Error::custom)
+            }
+        }
+
+        deserializer.deserialize_str(Visitor)
+    }
+}
 
 #[cfg(feature = "schemars")]
 impl schemars::JsonSchema for SerdePattern {
@@ -722,8 +718,8 @@ impl schemars::JsonSchema for SerdePattern {
         <String as schemars::JsonSchema>::schema_name()
     }
 
-    fn json_schema(gen: &mut schemars::gen::SchemaGenerator) -> schemars::schema::Schema {
-        <String as schemars::JsonSchema>::json_schema(gen)
+    fn json_schema(r#gen: &mut schemars::r#gen::SchemaGenerator) -> schemars::schema::Schema {
+        <String as schemars::JsonSchema>::json_schema(r#gen)
     }
 }
 
@@ -735,84 +731,6 @@ impl Deref for SerdePattern {
     }
 }
 
-#[derive(Debug, Clone, PartialEq)]
-#[cfg_attr(test, derive(Serialize))]
-pub struct DependencyGroups(BTreeMap<GroupName, Vec<DependencyGroupSpecifier>>);
-
-impl DependencyGroups {
-    /// Returns the names of the dependency groups.
-    pub fn keys(&self) -> impl Iterator<Item = &GroupName> {
-        self.0.keys()
-    }
-
-    /// Returns the dependency group with the given name.
-    pub fn get(&self, group: &GroupName) -> Option<&Vec<DependencyGroupSpecifier>> {
-        self.0.get(group)
-    }
-
-    /// Returns `true` if the dependency group is in the list.
-    pub fn contains_key(&self, group: &GroupName) -> bool {
-        self.0.contains_key(group)
-    }
-
-    /// Returns an iterator over the dependency groups.
-    pub fn iter(&self) -> impl Iterator<Item = (&GroupName, &Vec<DependencyGroupSpecifier>)> {
-        self.0.iter()
-    }
-}
-
-impl<'a> IntoIterator for &'a DependencyGroups {
-    type Item = (&'a GroupName, &'a Vec<DependencyGroupSpecifier>);
-    type IntoIter = std::collections::btree_map::Iter<'a, GroupName, Vec<DependencyGroupSpecifier>>;
-
-    fn into_iter(self) -> Self::IntoIter {
-        self.0.iter()
-    }
-}
-
-/// Ensure that all keys in the TOML table are unique.
-impl<'de> serde::de::Deserialize<'de> for DependencyGroups {
-    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
-    where
-        D: Deserializer<'de>,
-    {
-        struct GroupVisitor;
-
-        impl<'de> serde::de::Visitor<'de> for GroupVisitor {
-            type Value = DependencyGroups;
-
-            fn expecting(&self, formatter: &mut std::fmt::Formatter) -> std::fmt::Result {
-                formatter.write_str("a table with unique dependency group names")
-            }
-
-            fn visit_map<M>(self, mut access: M) -> Result<Self::Value, M::Error>
-            where
-                M: serde::de::MapAccess<'de>,
-            {
-                let mut sources = BTreeMap::new();
-                while let Some((key, value)) =
-                    access.next_entry::<GroupName, Vec<DependencyGroupSpecifier>>()?
-                {
-                    match sources.entry(key) {
-                        std::collections::btree_map::Entry::Occupied(entry) => {
-                            return Err(serde::de::Error::custom(format!(
-                                "duplicate dependency group: `{}`",
-                                entry.key()
-                            )));
-                        }
-                        std::collections::btree_map::Entry::Vacant(entry) => {
-                            entry.insert(value);
-                        }
-                    }
-                }
-                Ok(DependencyGroups(sources))
-            }
-        }
-
-        deserializer.deserialize_map(GroupVisitor)
-    }
-}
-
 #[derive(Serialize, Deserialize, Debug, Clone, PartialEq, Eq)]
 #[cfg_attr(feature = "schemars", derive(schemars::JsonSchema))]
 #[serde(rename_all = "kebab-case", try_from = "SourcesWire")]
@@ -1007,6 +925,13 @@ pub enum Source {
         path: PortablePathBuf,
         /// `false` by default.
         editable: Option<bool>,
+        /// Whether to treat the dependency as a buildable Python package (`true`) or as a virtual
+        /// package (`false`). If `false`, the package will not be built or installed, but its
+        /// dependencies will be included in the virtual environment.
+        ///
+        /// When omitted, the package status is inferred based on the presence of a `[build-system]`
+        /// in the project's `pyproject.toml`.
+        package: Option<bool>,
         #[serde(
             skip_serializing_if = "uv_pep508::marker::ser::is_empty",
             serialize_with = "uv_pep508::marker::ser::serialize",
@@ -1062,6 +987,7 @@ impl<'de> Deserialize<'de> for Source {
             url: Option<Url>,
             path: Option<PortablePathBuf>,
             editable: Option<bool>,
+            package: Option<bool>,
             index: Option<IndexName>,
             workspace: Option<bool>,
             #[serde(
@@ -1084,6 +1010,7 @@ impl<'de> Deserialize<'de> for Source {
             url,
             path,
             editable,
+            package,
             index,
             workspace,
             marker,
@@ -1125,6 +1052,11 @@ impl<'de> Deserialize<'de> for Source {
                     "cannot specify both `git` and `editable`",
                 ));
             }
+            if package.is_some() {
+                return Err(serde::de::Error::custom(
+                    "cannot specify both `git` and `package`",
+                ));
+            }
 
             // At most one of `rev`, `tag`, or `branch` may be set.
             match (rev.as_ref(), tag.as_ref(), branch.as_ref()) {
@@ -1135,7 +1067,7 @@ impl<'de> Deserialize<'de> for Source {
                 _ => {
                     return Err(serde::de::Error::custom(
                         "expected at most one of `rev`, `tag`, or `branch`",
-                    ))
+                    ));
                 }
             };
 
@@ -1200,6 +1132,11 @@ impl<'de> Deserialize<'de> for Source {
                     "cannot specify both `url` and `editable`",
                 ));
             }
+            if package.is_some() {
+                return Err(serde::de::Error::custom(
+                    "cannot specify both `url` and `package`",
+                ));
+            }
 
             return Ok(Self::Url {
                 url,
@@ -1248,9 +1185,17 @@ impl<'de> Deserialize<'de> for Source {
                 ));
             }
 
+            // A project must be packaged in order to be installed as editable.
+            if editable == Some(true) && package == Some(false) {
+                return Err(serde::de::Error::custom(
+                    "cannot specify both `editable = true` and `package = false`",
+                ));
+            }
+
             return Ok(Self::Path {
                 path,
                 editable,
+                package,
                 marker,
                 extra,
                 group,
@@ -1299,6 +1244,11 @@ impl<'de> Deserialize<'de> for Source {
                     "cannot specify both `index` and `editable`",
                 ));
             }
+            if package.is_some() {
+                return Err(serde::de::Error::custom(
+                    "cannot specify both `index` and `package`",
+                ));
+            }
 
             return Ok(Self::Registry {
                 index,
@@ -1350,6 +1300,11 @@ impl<'de> Deserialize<'de> for Source {
                     "cannot specify both `workspace` and `editable`",
                 ));
             }
+            if package.is_some() {
+                return Err(serde::de::Error::custom(
+                    "cannot specify both `workspace` and `package`",
+                ));
+            }
 
             return Ok(Self::Workspace {
                 workspace,
@@ -1376,23 +1331,36 @@ pub enum SourceError {
     WorkspacePackageUrl(String),
     #[error("Workspace dependency `{0}` must refer to local directory, not a file")]
     WorkspacePackageFile(String),
-    #[error("`{0}` did not resolve to a Git repository, but a Git reference (`--rev {1}`) was provided.")]
+    #[error(
+        "`{0}` did not resolve to a Git repository, but a Git reference (`--rev {1}`) was provided."
+    )]
     UnusedRev(String, String),
-    #[error("`{0}` did not resolve to a Git repository, but a Git reference (`--tag {1}`) was provided.")]
+    #[error(
+        "`{0}` did not resolve to a Git repository, but a Git reference (`--tag {1}`) was provided."
+    )]
     UnusedTag(String, String),
-    #[error("`{0}` did not resolve to a Git repository, but a Git reference (`--branch {1}`) was provided.")]
+    #[error(
+        "`{0}` did not resolve to a Git repository, but a Git reference (`--branch {1}`) was provided."
+    )]
     UnusedBranch(String, String),
-    #[error("`{0}` did not resolve to a local directory, but the `--editable` flag was provided. Editable installs are only supported for local directories.")]
+    #[error(
+        "`{0}` did not resolve to a local directory, but the `--editable` flag was provided. Editable installs are only supported for local directories."
+    )]
     UnusedEditable(String),
-    #[error("Workspace dependency `{0}` was marked as `--no-editable`, but workspace dependencies are always added in editable mode. Pass `--no-editable` to `uv sync` or `uv run` to install workspace dependencies in non-editable mode.")]
+    #[error(
+        "Workspace dependency `{0}` was marked as `--no-editable`, but workspace dependencies are always added in editable mode. Pass `--no-editable` to `uv sync` or `uv run` to install workspace dependencies in non-editable mode."
+    )]
     UnusedNoEditable(String),
     #[error("Failed to resolve absolute path")]
     Absolute(#[from] std::io::Error),
     #[error("Path contains invalid characters: `{}`", _0.display())]
     NonUtf8Path(PathBuf),
-    #[error("Source markers must be disjoint, but the following markers overlap: `{0}` and `{1}`.\n\n{hint}{colon} replace `{1}` with `{2}`.", hint = "hint".bold().cyan(), colon = ":".bold())]
+    #[error("Source markers must be disjoint, but the following markers overlap: `{0}` and `{1}`.\n\n{hint}{colon} replace `{1}` with `{2}`.", hint = "hint".bold().cyan(), colon = ":".bold()
+    )]
     OverlappingMarkers(String, String, String),
-    #[error("When multiple sources are provided, each source must include a platform marker (e.g., `marker = \"sys_platform == 'linux'\"`)")]
+    #[error(
+        "When multiple sources are provided, each source must include a platform marker (e.g., `marker = \"sys_platform == 'linux'\"`)"
+    )]
     MissingMarkers,
     #[error("Must provide at least one source")]
     EmptySources,
@@ -1479,6 +1447,7 @@ impl Source {
             RequirementSource::Path { install_path, .. }
             | RequirementSource::Directory { install_path, .. } => Source::Path {
                 editable,
+                package: None,
                 path: PortablePathBuf::from(
                     relative_to(&install_path, root)
                         .or_else(|_| std::path::absolute(&install_path))
@@ -1585,30 +1554,3 @@ pub enum DependencyType {
     /// A dependency in `dependency-groups.{0}`.
     Group(GroupName),
 }
-
-/// <https://github.com/serde-rs/serde/issues/1316#issue-332908452>
-mod serde_from_and_to_string {
-    use std::fmt::Display;
-    use std::str::FromStr;
-
-    use serde::{de, Deserialize, Deserializer, Serializer};
-
-    pub(super) fn serialize<T, S>(value: &T, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        T: Display,
-        S: Serializer,
-    {
-        serializer.collect_str(value)
-    }
-
-    pub(super) fn deserialize<'de, T, D>(deserializer: D) -> Result<T, D::Error>
-    where
-        T: FromStr,
-        T::Err: Display,
-        D: Deserializer<'de>,
-    {
-        String::deserialize(deserializer)?
-            .parse()
-            .map_err(de::Error::custom)
-    }
-}
