@@ -44,8 +44,11 @@ pub enum Error {
     MalformedWorkspace,
     #[error("Expected a dependency at index {0}")]
     MissingDependency(usize),
-    #[error("Cannot perform ambiguous update; found multiple entries with matching package names")]
-    Ambiguous,
+    #[error("Cannot perform ambiguous update; found multiple entries for `{}`:\n{}", package_name, requirements.iter().map(|requirement| format!("- `{requirement}`")).join("\n"))]
+    Ambiguous {
+        package_name: PackageName,
+        requirements: Vec<Requirement>,
+    },
 }
 
 /// The result of editing an array in a TOML document.
@@ -174,11 +177,10 @@ impl PyProjectTomlMut {
             .as_array_mut()
             .ok_or(Error::MalformedDependencies)?;
 
-        let name = req.name.clone();
         let edit = add_dependency(req, dependencies, source.is_some())?;
 
         if let Some(source) = source {
-            self.add_source(&name, source)?;
+            self.add_source(&req.name, source)?;
         }
 
         Ok(edit)
@@ -208,11 +210,10 @@ impl PyProjectTomlMut {
             .as_array_mut()
             .ok_or(Error::MalformedDependencies)?;
 
-        let name = req.name.clone();
         let edit = add_dependency(req, dev_dependencies, source.is_some())?;
 
         if let Some(source) = source {
-            self.add_source(&name, source)?;
+            self.add_source(&req.name, source)?;
         }
 
         Ok(edit)
@@ -411,7 +412,6 @@ impl PyProjectTomlMut {
             .as_array_mut()
             .ok_or(Error::MalformedDependencies)?;
 
-        let name = req.name.clone();
         let added = add_dependency(req, group, source.is_some())?;
 
         // If `project.optional-dependencies` is an inline table, reformat it.
@@ -427,7 +427,7 @@ impl PyProjectTomlMut {
         }
 
         if let Some(source) = source {
-            self.add_source(&name, source)?;
+            self.add_source(&req.name, source)?;
         }
 
         Ok(added)
@@ -463,7 +463,6 @@ impl PyProjectTomlMut {
             .as_array_mut()
             .ok_or(Error::MalformedDependencies)?;
 
-        let name = req.name.clone();
         let added = add_dependency(req, group, source.is_some())?;
 
         // To avoid churn in pyproject.toml, we only sort new group keys if the
@@ -485,7 +484,7 @@ impl PyProjectTomlMut {
         }
 
         if let Some(source) = source {
-            self.add_source(&name, source)?;
+            self.add_source(&req.name, source)?;
         }
 
         Ok(added)
@@ -865,7 +864,7 @@ impl PyProjectTomlMut {
                     let Some(dependencies) = dependencies.as_array() else {
                         continue;
                     };
-                    let Ok(extra) = ExtraName::new(extra.to_string()) else {
+                    let Ok(extra) = ExtraName::from_str(extra) else {
                         continue;
                     };
 
@@ -882,7 +881,7 @@ impl PyProjectTomlMut {
                 let Some(dependencies) = dependencies.as_array() else {
                     continue;
                 };
-                let Ok(group) = GroupName::new(group.to_string()) else {
+                let Ok(group) = GroupName::from_str(group) else {
                     continue;
                 };
 
@@ -1128,7 +1127,13 @@ pub fn add_dependency(
             Ok(ArrayEdit::Update(i))
         }
         // Cannot perform ambiguous updates.
-        _ => Err(Error::Ambiguous),
+        _ => Err(Error::Ambiguous {
+            package_name: req.name.clone(),
+            requirements: to_replace
+                .into_iter()
+                .map(|(_, requirement)| requirement)
+                .collect(),
+        }),
     }
 }
 
