@@ -1,11 +1,13 @@
 //! Resolve the current [`ProjectWorkspace`] or [`Workspace`].
 
-use std::collections::BTreeMap;
+use std::collections::{BTreeMap, BTreeSet};
 use std::path::{Path, PathBuf};
+use std::sync::{Arc, Mutex};
 
 use glob::{glob, GlobError, PatternError};
-use rustc_hash::FxHashSet;
+use rustc_hash::{FxHashMap, FxHashSet};
 use tracing::{debug, trace, warn};
+
 use uv_distribution_types::Index;
 use uv_fs::{Simplified, CWD};
 use uv_normalize::{GroupName, PackageName, DEV_DEPENDENCIES};
@@ -22,6 +24,24 @@ use crate::pyproject::{
     Project, PyProjectToml, PyprojectTomlError, Sources, ToolUvSources, ToolUvWorkspace,
 };
 
+type WorkspaceMembers = Arc<BTreeMap<PackageName, WorkspaceMember>>;
+
+/// Cache key for workspace discovery.
+///
+/// Given this key, the discovered workspace member list is the same.
+#[derive(Debug, Default, Clone, Hash, PartialEq, Eq)]
+struct WorkspaceCacheKey {
+    workspace_root: PathBuf,
+    discovery_options: DiscoveryOptions,
+}
+
+/// Cache for workspace discovery.
+///
+/// Avoid re-reading the `pyproject.toml` files in a workspace for each member by caching the
+/// workspace members by their workspace root.
+#[derive(Debug, Default, Clone)]
+pub struct WorkspaceCache(Arc<Mutex<FxHashMap<WorkspaceCacheKey, WorkspaceMembers>>>);
+
 #[derive(thiserror::Error, Debug)]
 pub enum WorkspaceError {
     // Workspace structure errors.
@@ -58,23 +78,23 @@ pub enum WorkspaceError {
     Normalize(#[source] std::io::Error),
 }
 
-#[derive(Debug, Default, Clone)]
-pub enum MemberDiscovery<'a> {
+#[derive(Debug, Default, Clone, Hash, PartialEq, Eq)]
+pub enum MemberDiscovery {
     /// Discover all workspace members.
     #[default]
     All,
     /// Don't discover any workspace members.
     None,
     /// Discover workspace members, but ignore the given paths.
-    Ignore(FxHashSet<&'a Path>),
+    Ignore(BTreeSet<PathBuf>),
 }
 
-#[derive(Debug, Default, Clone)]
-pub struct DiscoveryOptions<'a> {
+#[derive(Debug, Default, Clone, Hash, PartialEq, Eq)]
+pub struct DiscoveryOptions {
     /// The path to stop discovery at.
-    pub stop_discovery_at: Option<&'a Path>,
+    pub stop_discovery_at: Option<PathBuf>,
     /// The strategy to use when discovering workspace members.
-    pub members: MemberDiscovery<'a>,
+    pub members: MemberDiscovery,
 }
 
 /// A workspace, consisting of a root directory and members. See [`ProjectWorkspace`].
@@ -87,7 +107,7 @@ pub struct Workspace {
     /// the `uv.tool.workspace`, or the `pyproject.toml` in an implicit single workspace project.
     install_path: PathBuf,
     /// The members of the workspace.
-    packages: BTreeMap<PackageName, WorkspaceMember>,
+    packages: WorkspaceMembers,
     /// The sources table from the workspace `pyproject.toml`.
     ///
     /// This table is overridden by the project sources.
@@ -124,7 +144,8 @@ impl Workspace {
     /// ```
     pub async fn discover(
         path: &Path,
-        options: &DiscoveryOptions<'_>,
+        options: &DiscoveryOptions,
+        cache: &WorkspaceCache,
     ) -> Result<Workspace, WorkspaceError> {
         let path = std::path::absolute(path)
             .map_err(WorkspaceError::Normalize)?
@@ -211,6 +232,7 @@ impl Workspace {
             workspace_pyproject_toml,
             current_project,
             options,
+            cache,
         )
         .await
     }
@@ -237,7 +259,7 @@ impl Workspace {
         pyproject_toml: PyProjectToml,
     ) -> Option<Self> {
         let mut packages = self.packages;
-        let member = packages.get_mut(package_name)?;
+        let member = Arc::make_mut(&mut packages).get_mut(package_name)?;
 
         if member.root == self.install_path {
             // If the member is also the workspace root, update _both_ the member entry and the
@@ -656,57 +678,113 @@ impl Workspace {
         workspace_definition: ToolUvWorkspace,
         workspace_pyproject_toml: PyProjectToml,
         current_project: Option<WorkspaceMember>,
-        options: &DiscoveryOptions<'_>,
+        options: &DiscoveryOptions,
+        cache: &WorkspaceCache,
     ) -> Result<Workspace, WorkspaceError> {
+        let cache_key = WorkspaceCacheKey {
+            workspace_root: workspace_root.clone(),
+            discovery_options: options.clone(),
+        };
+        let cache_entry = {
+            // Acquire the lock for the minimal required region
+            let cache = cache.0.lock().expect("there was a panic in another thread");
+            cache.get(&cache_key).cloned()
+        };
+        let mut workspace_members = if let Some(workspace_members) = cache_entry {
+            trace!(
+                "Cached workspace members for: `{}`",
+                &workspace_root.simplified_display()
+            );
+            workspace_members
+        } else {
+            trace!(
+                "Discovering workspace members for: `{}`",
+                &workspace_root.simplified_display()
+            );
+            let workspace_members = Self::collect_members_only(
+                &workspace_root,
+                &workspace_definition,
+                &workspace_pyproject_toml,
+                options,
+            )
+            .await?;
+            {
+                // Acquire the lock for the minimal required region
+                let mut cache = cache.0.lock().expect("there was a panic in another thread");
+                cache.insert(cache_key, Arc::new(workspace_members.clone()));
+            }
+            Arc::new(workspace_members)
+        };
+
+        // For the cases such as `MemberDiscovery::None`, add the current project if missing.
+        if let Some(root_member) = current_project {
+            if !workspace_members.contains_key(&root_member.project.name) {
+                debug!(
+                    "Adding current workspace member: `{}`",
+                    root_member.root.simplified_display()
+                );
+
+                Arc::make_mut(&mut workspace_members)
+                    .insert(root_member.project.name.clone(), root_member);
+            }
+        }
+
+        let workspace_sources = workspace_pyproject_toml
+            .tool
+            .clone()
+            .and_then(|tool| tool.uv)
+            .and_then(|uv| uv.sources)
+            .map(ToolUvSources::into_inner)
+            .unwrap_or_default();
+
+        let workspace_indexes = workspace_pyproject_toml
+            .tool
+            .clone()
+            .and_then(|tool| tool.uv)
+            .and_then(|uv| uv.index)
+            .unwrap_or_default();
+
+        Ok(Workspace {
+            install_path: workspace_root,
+            packages: workspace_members,
+            sources: workspace_sources,
+            indexes: workspace_indexes,
+            pyproject_toml: workspace_pyproject_toml,
+        })
+    }
+
+    async fn collect_members_only(
+        workspace_root: &PathBuf,
+        workspace_definition: &ToolUvWorkspace,
+        workspace_pyproject_toml: &PyProjectToml,
+        options: &DiscoveryOptions,
+    ) -> Result<BTreeMap<PackageName, WorkspaceMember>, WorkspaceError> {
         let mut workspace_members = BTreeMap::new();
         // Avoid reading a `pyproject.toml` more than once.
         let mut seen = FxHashSet::default();
 
         // Add the project at the workspace root, if it exists and if it's distinct from the current
-        // project.
-        if current_project
-            .as_ref()
-            .map(|root_member| root_member.root != workspace_root)
-            .unwrap_or(true)
-        {
-            if let Some(project) = &workspace_pyproject_toml.project {
-                let pyproject_path = workspace_root.join("pyproject.toml");
-                let contents = fs_err::read_to_string(&pyproject_path)?;
-                let pyproject_toml = PyProjectToml::from_string(contents)
-                    .map_err(|err| WorkspaceError::Toml(pyproject_path.clone(), Box::new(err)))?;
+        // project. If it is the current project, it is added as such in the next step.
+        if let Some(project) = &workspace_pyproject_toml.project {
+            let pyproject_path = workspace_root.join("pyproject.toml");
+            let contents = fs_err::read_to_string(&pyproject_path)?;
+            let pyproject_toml = PyProjectToml::from_string(contents)
+                .map_err(|err| WorkspaceError::Toml(pyproject_path.clone(), Box::new(err)))?;
 
-                debug!(
-                    "Adding root workspace member: `{}`",
-                    workspace_root.simplified_display()
-                );
-
-                seen.insert(workspace_root.clone());
-                if let Some(existing) = workspace_members.insert(
-                    project.name.clone(),
-                    WorkspaceMember {
-                        root: workspace_root.clone(),
-                        project: project.clone(),
-                        pyproject_toml,
-                    },
-                ) {
-                    return Err(WorkspaceError::DuplicatePackage {
-                        name: project.name.clone(),
-                        first: existing.root.clone(),
-                        second: workspace_root,
-                    });
-                }
-            };
-        }
-
-        // The current project is a workspace member, especially in a single project workspace.
-        if let Some(root_member) = current_project {
             debug!(
-                "Adding current workspace member: `{}`",
-                root_member.root.simplified_display()
+                "Adding root workspace member: `{}`",
+                workspace_root.simplified_display()
             );
 
-            seen.insert(root_member.root.clone());
-            workspace_members.insert(root_member.project.name.clone(), root_member);
+            seen.insert(workspace_root.clone());
+            workspace_members.insert(
+                project.name.clone(),
+                WorkspaceMember {
+                    root: workspace_root.clone(),
+                    project: project.clone(),
+                    pyproject_toml,
+                },
+            );
         }
 
         // Add all other workspace members.
@@ -744,8 +822,7 @@ impl Workspace {
                 }
 
                 // If the member is excluded, ignore it.
-                if is_excluded_from_workspace(&member_root, &workspace_root, &workspace_definition)?
-                {
+                if is_excluded_from_workspace(&member_root, workspace_root, workspace_definition)? {
                     debug!(
                         "Ignoring workspace member: `{}`",
                         member_root.simplified_display()
@@ -762,34 +839,38 @@ impl Workspace {
                 let pyproject_path = member_root.join("pyproject.toml");
                 let contents = match fs_err::tokio::read_to_string(&pyproject_path).await {
                     Ok(contents) => contents,
-                    Err(err) if err.kind() == std::io::ErrorKind::NotFound => {
-                        // If the directory is hidden, skip it.
-                        if member_root
-                            .file_name()
-                            .map(|name| name.as_encoded_bytes().starts_with(b"."))
-                            .unwrap_or(false)
-                        {
-                            debug!(
-                                "Ignoring hidden workspace member: `{}`",
+                    Err(err) => {
+                        if !fs_err::metadata(&member_root)?.is_dir() {
+                            warn!(
+                                "Ignoring non-directory workspace member: `{}`",
                                 member_root.simplified_display()
                             );
                             continue;
                         }
 
-                        return Err(WorkspaceError::MissingPyprojectTomlMember(
-                            member_root,
-                            member_glob.to_string(),
-                        ));
-                    }
-                    Err(err) if err.kind() == std::io::ErrorKind::NotADirectory => {
-                        warn!(
-                            "Ignoring non-directory workspace member: `{}`",
-                            member_root.simplified_display()
-                        );
+                        // A directory exists, but it doesn't contain a `pyproject.toml`.
+                        if err.kind() == std::io::ErrorKind::NotFound {
+                            // If the directory is hidden, skip it.
+                            if member_root
+                                .file_name()
+                                .map(|name| name.as_encoded_bytes().starts_with(b"."))
+                                .unwrap_or(false)
+                            {
+                                debug!(
+                                    "Ignoring hidden workspace member: `{}`",
+                                    member_root.simplified_display()
+                                );
+                                continue;
+                            }
 
-                        continue;
+                            return Err(WorkspaceError::MissingPyprojectTomlMember(
+                                member_root,
+                                member_glob.to_string(),
+                            ));
+                        }
+
+                        return Err(err.into());
                     }
-                    Err(err) => return Err(err.into()),
                 };
                 let pyproject_toml = PyProjectToml::from_string(contents)
                     .map_err(|err| WorkspaceError::Toml(pyproject_path.clone(), Box::new(err)))?;
@@ -838,7 +919,7 @@ impl Workspace {
 
         // Test for nested workspaces.
         for member in workspace_members.values() {
-            if member.root() != &workspace_root
+            if member.root() != workspace_root
                 && member
                     .pyproject_toml
                     .tool
@@ -850,34 +931,12 @@ impl Workspace {
                 return Err(WorkspaceError::NestedWorkspace(member.root.clone()));
             }
         }
-
-        let workspace_sources = workspace_pyproject_toml
-            .tool
-            .clone()
-            .and_then(|tool| tool.uv)
-            .and_then(|uv| uv.sources)
-            .map(ToolUvSources::into_inner)
-            .unwrap_or_default();
-
-        let workspace_indexes = workspace_pyproject_toml
-            .tool
-            .clone()
-            .and_then(|tool| tool.uv)
-            .and_then(|uv| uv.index)
-            .unwrap_or_default();
-
-        Ok(Workspace {
-            install_path: workspace_root,
-            packages: workspace_members,
-            sources: workspace_sources,
-            indexes: workspace_indexes,
-            pyproject_toml: workspace_pyproject_toml,
-        })
+        Ok(workspace_members)
     }
 }
 
 /// A project in a workspace.
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, PartialEq)]
 #[cfg_attr(test, derive(serde::Serialize))]
 pub struct WorkspaceMember {
     /// The path to the project root.
@@ -1002,7 +1061,8 @@ impl ProjectWorkspace {
     /// only directories between the current path and `stop_discovery_at` are considered.
     pub async fn discover(
         path: &Path,
-        options: &DiscoveryOptions<'_>,
+        options: &DiscoveryOptions,
+        cache: &WorkspaceCache,
     ) -> Result<Self, WorkspaceError> {
         let project_root = path
             .ancestors()
@@ -1010,6 +1070,7 @@ impl ProjectWorkspace {
                 // Only walk up the given directory, if any.
                 options
                     .stop_discovery_at
+                    .as_deref()
                     .and_then(Path::parent)
                     .map(|stop_discovery_at| stop_discovery_at != *path)
                     .unwrap_or(true)
@@ -1022,13 +1083,14 @@ impl ProjectWorkspace {
             project_root.simplified_display()
         );
 
-        Self::from_project_root(project_root, options).await
+        Self::from_project_root(project_root, options, cache).await
     }
 
     /// Discover the workspace starting from the directory containing the `pyproject.toml`.
     async fn from_project_root(
         project_root: &Path,
-        options: &DiscoveryOptions<'_>,
+        options: &DiscoveryOptions,
+        cache: &WorkspaceCache,
     ) -> Result<Self, WorkspaceError> {
         // Read the current `pyproject.toml`.
         let pyproject_path = project_root.join("pyproject.toml");
@@ -1042,14 +1104,15 @@ impl ProjectWorkspace {
             .clone()
             .ok_or(WorkspaceError::MissingProject(pyproject_path))?;
 
-        Self::from_project(project_root, &project, &pyproject_toml, options).await
+        Self::from_project(project_root, &project, &pyproject_toml, options, cache).await
     }
 
     /// If the current directory contains a `pyproject.toml` with a `project` table, discover the
     /// workspace and return it, otherwise it is a dynamic path dependency and we return `Ok(None)`.
     pub async fn from_maybe_project_root(
         install_path: &Path,
-        options: &DiscoveryOptions<'_>,
+        options: &DiscoveryOptions,
+        cache: &WorkspaceCache,
     ) -> Result<Option<Self>, WorkspaceError> {
         // Read the `pyproject.toml`.
         let pyproject_path = install_path.join("pyproject.toml");
@@ -1066,7 +1129,7 @@ impl ProjectWorkspace {
             return Ok(None);
         };
 
-        match Self::from_project(install_path, &project, &pyproject_toml, options).await {
+        match Self::from_project(install_path, &project, &pyproject_toml, options, cache).await {
             Ok(workspace) => Ok(Some(workspace)),
             Err(WorkspaceError::NonWorkspace(_)) => Ok(None),
             Err(err) => Err(err),
@@ -1112,7 +1175,8 @@ impl ProjectWorkspace {
         install_path: &Path,
         project: &Project,
         project_pyproject_toml: &PyProjectToml,
-        options: &DiscoveryOptions<'_>,
+        options: &DiscoveryOptions,
+        cache: &WorkspaceCache,
     ) -> Result<Self, WorkspaceError> {
         let project_path = std::path::absolute(install_path)
             .map_err(WorkspaceError::Normalize)?
@@ -1162,8 +1226,10 @@ impl ProjectWorkspace {
             // above it, so the project is an implicit workspace root identical to the project root.
             debug!("No workspace root found, using project root");
 
-            let current_project_as_members =
-                BTreeMap::from_iter([(project.name.clone(), current_project)]);
+            let current_project_as_members = Arc::new(BTreeMap::from_iter([(
+                project.name.clone(),
+                current_project,
+            )]));
             return Ok(Self {
                 project_root: project_path.clone(),
                 project_name: project.name.clone(),
@@ -1190,6 +1256,7 @@ impl ProjectWorkspace {
             workspace_pyproject_toml,
             Some(current_project),
             options,
+            cache,
         )
         .await?;
 
@@ -1204,7 +1271,7 @@ impl ProjectWorkspace {
 /// Find the workspace root above the current project, if any.
 async fn find_workspace(
     project_root: &Path,
-    options: &DiscoveryOptions<'_>,
+    options: &DiscoveryOptions,
 ) -> Result<Option<(PathBuf, ToolUvWorkspace, PyProjectToml)>, WorkspaceError> {
     // Skip 1 to ignore the current project itself.
     for workspace_root in project_root
@@ -1213,6 +1280,7 @@ async fn find_workspace(
             // Only walk up the given directory, if any.
             options
                 .stop_discovery_at
+                .as_deref()
                 .and_then(Path::parent)
                 .map(|stop_discovery_at| stop_discovery_at != *path)
                 .unwrap_or(true)
@@ -1362,7 +1430,8 @@ impl VirtualProject {
     /// discovering the main workspace.
     pub async fn discover(
         path: &Path,
-        options: &DiscoveryOptions<'_>,
+        options: &DiscoveryOptions,
+        cache: &WorkspaceCache,
     ) -> Result<Self, WorkspaceError> {
         assert!(
             path.is_absolute(),
@@ -1374,6 +1443,7 @@ impl VirtualProject {
                 // Only walk up the given directory, if any.
                 options
                     .stop_discovery_at
+                    .as_deref()
                     .and_then(Path::parent)
                     .map(|stop_discovery_at| stop_discovery_at != *path)
                     .unwrap_or(true)
@@ -1394,9 +1464,14 @@ impl VirtualProject {
 
         if let Some(project) = pyproject_toml.project.as_ref() {
             // If the `pyproject.toml` contains a `[project]` table, it's a project.
-            let project =
-                ProjectWorkspace::from_project(project_root, project, &pyproject_toml, options)
-                    .await?;
+            let project = ProjectWorkspace::from_project(
+                project_root,
+                project,
+                &pyproject_toml,
+                options,
+                cache,
+            )
+            .await?;
             Ok(Self::Project(project))
         } else if let Some(workspace) = pyproject_toml
             .tool
@@ -1416,6 +1491,7 @@ impl VirtualProject {
                 pyproject_toml,
                 None,
                 options,
+                cache,
             )
             .await?;
 
@@ -1496,10 +1572,11 @@ mod tests {
     use insta::{assert_json_snapshot, assert_snapshot};
 
     use uv_normalize::GroupName;
+    use uv_pypi_types::DependencyGroupSpecifier;
 
-    use crate::pyproject::{DependencyGroupSpecifier, PyProjectToml};
+    use crate::pyproject::PyProjectToml;
     use crate::workspace::{DiscoveryOptions, ProjectWorkspace};
-    use crate::WorkspaceError;
+    use crate::{WorkspaceCache, WorkspaceError};
 
     async fn workspace_test(folder: &str) -> (ProjectWorkspace, String) {
         let root_dir = env::current_dir()
@@ -1510,10 +1587,13 @@ mod tests {
             .unwrap()
             .join("scripts")
             .join("workspaces");
-        let project =
-            ProjectWorkspace::discover(&root_dir.join(folder), &DiscoveryOptions::default())
-                .await
-                .unwrap();
+        let project = ProjectWorkspace::discover(
+            &root_dir.join(folder),
+            &DiscoveryOptions::default(),
+            &WorkspaceCache::default(),
+        )
+        .await
+        .unwrap();
         let root_escaped = regex::escape(root_dir.to_string_lossy().as_ref());
         (project, root_escaped)
     }
@@ -1522,9 +1602,13 @@ mod tests {
         folder: &Path,
     ) -> Result<(ProjectWorkspace, String), (WorkspaceError, String)> {
         let root_escaped = regex::escape(folder.to_string_lossy().as_ref());
-        let project = ProjectWorkspace::discover(folder, &DiscoveryOptions::default())
-            .await
-            .map_err(|error| (error, root_escaped.clone()))?;
+        let project = ProjectWorkspace::discover(
+            folder,
+            &DiscoveryOptions::default(),
+            &WorkspaceCache::default(),
+        )
+        .await
+        .map_err(|error| (error, root_escaped.clone()))?;
 
         Ok((project, root_escaped))
     }
@@ -1898,6 +1982,7 @@ mod tests {
         "###);
         });
     }
+
     #[tokio::test]
     async fn exclude_package() -> Result<()> {
         let root = tempfile::TempDir::new()?;
