@@ -3,7 +3,7 @@ use anyhow::Context;
 use owo_colors::OwoColorize;
 use std::borrow::Cow;
 use std::io::stdout;
-use std::path::Path;
+use std::path::{Path, PathBuf};
 use std::time::Duration;
 use std::{fmt::Display, fmt::Write, process::ExitCode};
 
@@ -47,11 +47,13 @@ pub(crate) use tool::uninstall::uninstall as tool_uninstall;
 pub(crate) use tool::update_shell::update_shell as tool_update_shell;
 pub(crate) use tool::upgrade::upgrade as tool_upgrade;
 use uv_cache::Cache;
+use uv_configuration::Concurrency;
 use uv_distribution_types::InstalledMetadata;
 use uv_fs::{Simplified, CWD};
 use uv_installer::compile_tree;
 use uv_normalize::PackageName;
 use uv_python::PythonEnvironment;
+use uv_scripts::Pep723Script;
 pub(crate) use venv::venv;
 pub(crate) use version::version;
 
@@ -147,6 +149,7 @@ pub(super) struct DryRunEvent<T: Display> {
 /// See the `--compile` option on `pip sync` and `pip install`.
 pub(super) async fn compile_bytecode(
     venv: &PythonEnvironment,
+    concurrency: &Concurrency,
     cache: &Cache,
     printer: Printer,
 ) -> anyhow::Result<()> {
@@ -154,14 +157,19 @@ pub(super) async fn compile_bytecode(
     let mut files = 0;
     for site_packages in venv.site_packages() {
         let site_packages = CWD.join(site_packages);
-        files += compile_tree(&site_packages, venv.python_executable(), cache.root())
-            .await
-            .with_context(|| {
-                format!(
-                    "Failed to bytecode-compile Python file in: {}",
-                    site_packages.user_display()
-                )
-            })?;
+        files += compile_tree(
+            &site_packages,
+            venv.python_executable(),
+            concurrency,
+            cache.root(),
+        )
+        .await
+        .with_context(|| {
+            format!(
+                "Failed to bytecode-compile Python file in: {}",
+                site_packages.user_display()
+            )
+        })?;
     }
     let s = if files == 1 { "" } else { "s" };
     writeln!(
@@ -283,3 +291,12 @@ pub(super) fn capitalize(s: &str) -> String {
         Some(c) => c.to_uppercase().collect::<String>() + chars.as_str(),
     }
 }
+
+/// A Python file that may or may not include an existing PEP 723 script tag.
+#[derive(Debug)]
+pub(crate) enum ScriptPath {
+    /// The Python file already includes a PEP 723 script tag.
+    Script(Pep723Script),
+    /// The Python file does not include a PEP 723 script tag.
+    Path(PathBuf),
+}
