@@ -4,19 +4,18 @@ use anstream::print;
 use anyhow::{Error, Result};
 use futures::StreamExt;
 use tokio::sync::Semaphore;
+use uv_auth::UrlAuthPolicies;
 use uv_cache::{Cache, Refresh};
 use uv_cache_info::Timestamp;
-use uv_client::{Connectivity, RegistryClientBuilder};
-use uv_configuration::{
-    Concurrency, DevGroupsSpecification, PreviewMode, TargetTriple, TrustedHost,
-};
+use uv_client::RegistryClientBuilder;
+use uv_configuration::{Concurrency, DependencyGroups, PreviewMode, TargetTriple};
 use uv_distribution_types::IndexCapabilities;
 use uv_pep508::PackageName;
 use uv_python::{PythonDownloads, PythonPreference, PythonRequest, PythonVersion};
 use uv_resolver::{PackageMap, TreeDisplay};
 use uv_scripts::{Pep723ItemRef, Pep723Script};
 use uv_settings::PythonInstallMirrors;
-use uv_workspace::{DiscoveryOptions, Workspace};
+use uv_workspace::{DiscoveryOptions, Workspace, WorkspaceCache};
 
 use crate::commands::pip::latest::LatestClient;
 use crate::commands::pip::loggers::DefaultResolveLogger;
@@ -29,13 +28,13 @@ use crate::commands::project::{
 use crate::commands::reporters::LatestVersionReporter;
 use crate::commands::{diagnostics, ExitStatus};
 use crate::printer::Printer;
-use crate::settings::ResolverSettings;
+use crate::settings::{NetworkSettings, ResolverSettings};
 
 /// Run a command.
 #[allow(clippy::fn_params_excessive_bools)]
 pub(crate) async fn tree(
     project_dir: &Path,
-    dev: DevGroupsSpecification,
+    dev: DependencyGroups,
     locked: bool,
     frozen: bool,
     universal: bool,
@@ -50,24 +49,25 @@ pub(crate) async fn tree(
     python: Option<String>,
     install_mirrors: PythonInstallMirrors,
     settings: ResolverSettings,
+    network_settings: &NetworkSettings,
     script: Option<Pep723Script>,
     python_preference: PythonPreference,
     python_downloads: PythonDownloads,
-    connectivity: Connectivity,
     concurrency: Concurrency,
-    native_tls: bool,
-    allow_insecure_host: &[TrustedHost],
     no_config: bool,
     cache: &Cache,
     printer: Printer,
     preview: PreviewMode,
 ) -> Result<ExitStatus> {
     // Find the project requirements.
+    let workspace_cache = WorkspaceCache::default();
     let workspace;
     let target = if let Some(script) = script.as_ref() {
         LockTarget::Script(script)
     } else {
-        workspace = Workspace::discover(project_dir, &DiscoveryOptions::default()).await?;
+        workspace =
+            Workspace::discover(project_dir, &DiscoveryOptions::default(), &workspace_cache)
+                .await?;
         LockTarget::Workspace(&workspace)
     };
 
@@ -77,6 +77,8 @@ pub(crate) async fn tree(
         LockTarget::Script(_) => vec![],
     };
 
+    let native_tls = network_settings.native_tls;
+
     // Find an interpreter for the project, unless `--frozen` and `--universal` are both set.
     let interpreter = if frozen && universal {
         None
@@ -85,11 +87,9 @@ pub(crate) async fn tree(
             LockTarget::Script(script) => ScriptInterpreter::discover(
                 Pep723ItemRef::Script(script),
                 python.as_deref().map(PythonRequest::parse),
+                network_settings,
                 python_preference,
                 python_downloads,
-                connectivity,
-                native_tls,
-                allow_insecure_host,
                 &install_mirrors,
                 no_config,
                 Some(false),
@@ -102,11 +102,9 @@ pub(crate) async fn tree(
                 workspace,
                 project_dir,
                 python.as_deref().map(PythonRequest::parse),
+                network_settings,
                 python_preference,
                 python_downloads,
-                connectivity,
-                native_tls,
-                allow_insecure_host,
                 &install_mirrors,
                 no_config,
                 Some(false),
@@ -138,12 +136,10 @@ pub(crate) async fn tree(
         mode,
         target,
         settings.as_ref(),
+        network_settings,
         &state,
         Box::new(DefaultResolveLogger),
-        connectivity,
         concurrency,
-        native_tls,
-        allow_insecure_host,
         cache,
         printer,
         preview,
@@ -188,7 +184,7 @@ pub(crate) async fn tree(
             PackageMap::default()
         } else {
             let ResolverSettings {
-                index_locations: _,
+                index_locations,
                 index_strategy: _,
                 keyring_provider,
                 resolution: _,
@@ -211,10 +207,11 @@ pub(crate) async fn tree(
             let client = RegistryClientBuilder::new(
                 cache.clone().with_refresh(Refresh::All(Timestamp::now())),
             )
-            .native_tls(native_tls)
-            .connectivity(connectivity)
+            .native_tls(network_settings.native_tls)
+            .connectivity(network_settings.connectivity)
+            .allow_insecure_host(network_settings.allow_insecure_host.clone())
+            .url_auth_policies(UrlAuthPolicies::from(index_locations))
             .keyring(*keyring_provider)
-            .allow_insecure_host(allow_insecure_host.to_vec())
             .build();
             let download_concurrency = Semaphore::new(concurrency.downloads);
 
