@@ -7,8 +7,8 @@ use tracing::{debug, trace};
 
 use uv_cache::{Cache, Refresh};
 use uv_cache_info::Timestamp;
-use uv_client::{BaseClientBuilder, Connectivity};
-use uv_configuration::{Concurrency, DryRun, PreviewMode, Reinstall, TrustedHost, Upgrade};
+use uv_client::BaseClientBuilder;
+use uv_configuration::{Concurrency, DryRun, PreviewMode, Reinstall, Upgrade};
 use uv_distribution_types::{NameRequirementSpecification, UnresolvedRequirementSpecification};
 use uv_normalize::PackageName;
 use uv_pep440::{VersionSpecifier, VersionSpecifiers};
@@ -21,9 +21,9 @@ use uv_requirements::{RequirementsSource, RequirementsSpecification};
 use uv_settings::{PythonInstallMirrors, ResolverInstallerOptions, ToolOptions};
 use uv_tool::InstalledTools;
 use uv_warnings::warn_user;
+use uv_workspace::WorkspaceCache;
 
 use crate::commands::pip::loggers::{DefaultInstallLogger, DefaultResolveLogger};
-
 use crate::commands::pip::operations::Modifications;
 use crate::commands::project::{
     resolve_environment, resolve_names, sync_environment, update_environment,
@@ -34,7 +34,7 @@ use crate::commands::tool::{Target, ToolRequest};
 use crate::commands::ExitStatus;
 use crate::commands::{diagnostics, reporters::PythonDownloadReporter};
 use crate::printer::Printer;
-use crate::settings::ResolverInstallerSettings;
+use crate::settings::{NetworkSettings, ResolverInstallerSettings};
 
 /// Install a tool.
 #[allow(clippy::fn_params_excessive_bools)]
@@ -50,21 +50,19 @@ pub(crate) async fn install(
     force: bool,
     options: ResolverInstallerOptions,
     settings: ResolverInstallerSettings,
+    network_settings: NetworkSettings,
     python_preference: PythonPreference,
     python_downloads: PythonDownloads,
     installer_metadata: bool,
-    connectivity: Connectivity,
     concurrency: Concurrency,
-    native_tls: bool,
-    allow_insecure_host: &[TrustedHost],
     cache: Cache,
     printer: Printer,
     preview: PreviewMode,
 ) -> Result<ExitStatus> {
     let client_builder = BaseClientBuilder::new()
-        .connectivity(connectivity)
-        .native_tls(native_tls)
-        .allow_insecure_host(allow_insecure_host.to_vec());
+        .connectivity(network_settings.connectivity)
+        .native_tls(network_settings.native_tls)
+        .allow_insecure_host(network_settings.allow_insecure_host.clone());
 
     let reporter = PythonDownloadReporter::single(printer);
 
@@ -88,11 +86,12 @@ pub(crate) async fn install(
 
     // Initialize any shared state.
     let state = PlatformState::default();
+    let workspace_cache = WorkspaceCache::default();
 
     let client_builder = BaseClientBuilder::new()
-        .connectivity(connectivity)
-        .native_tls(native_tls)
-        .allow_insecure_host(allow_insecure_host.to_vec());
+        .connectivity(network_settings.connectivity)
+        .native_tls(network_settings.native_tls)
+        .allow_insecure_host(network_settings.allow_insecure_host.clone());
 
     // Parse the input requirement.
     let request = ToolRequest::parse(&package, from.as_deref());
@@ -131,12 +130,11 @@ pub(crate) async fn install(
                 requirement,
                 &interpreter,
                 &settings,
+                &network_settings,
                 &state,
-                connectivity,
                 concurrency,
-                native_tls,
-                allow_insecure_host,
                 &cache,
+                &workspace_cache,
                 printer,
                 preview,
             )
@@ -245,12 +243,11 @@ pub(crate) async fn install(
                 spec.requirements.clone(),
                 &interpreter,
                 &settings,
+                &network_settings,
                 &state,
-                connectivity,
                 concurrency,
-                native_tls,
-                allow_insecure_host,
                 &cache,
+                &workspace_cache,
                 printer,
                 preview,
             )
@@ -271,12 +268,11 @@ pub(crate) async fn install(
         spec.overrides,
         &interpreter,
         &settings,
+        &network_settings,
         &state,
-        connectivity,
         concurrency,
-        native_tls,
-        allow_insecure_host,
         &cache,
+        &workspace_cache,
         printer,
         preview,
     )
@@ -401,15 +397,14 @@ pub(crate) async fn install(
             spec,
             Modifications::Exact,
             &settings,
+            &network_settings,
             &state,
             Box::new(DefaultResolveLogger),
             Box::new(DefaultInstallLogger),
             installer_metadata,
-            connectivity,
             concurrency,
-            native_tls,
-            allow_insecure_host,
             &cache,
+            workspace_cache,
             DryRun::Disabled,
             printer,
             preview,
@@ -418,7 +413,7 @@ pub(crate) async fn install(
         {
             Ok(update) => update.into_environment(),
             Err(ProjectError::Operation(err)) => {
-                return diagnostics::OperationDiagnostic::native_tls(native_tls)
+                return diagnostics::OperationDiagnostic::native_tls(network_settings.native_tls)
                     .report(err)
                     .map_or(Ok(ExitStatus::Failure), |err| Err(err.into()))
             }
@@ -441,12 +436,10 @@ pub(crate) async fn install(
             spec.clone(),
             &interpreter,
             settings.as_ref().into(),
+            &network_settings,
             &state,
             Box::new(DefaultResolveLogger),
-            connectivity,
             concurrency,
-            native_tls,
-            allow_insecure_host,
             &cache,
             printer,
             preview,
@@ -478,9 +471,11 @@ pub(crate) async fn install(
                     .await
                     .ok()
                     .flatten() else {
-                        return diagnostics::OperationDiagnostic::native_tls(native_tls)
-                            .report(err)
-                            .map_or(Ok(ExitStatus::Failure), |err| Err(err.into()));
+                        return diagnostics::OperationDiagnostic::native_tls(
+                            network_settings.native_tls,
+                        )
+                        .report(err)
+                        .map_or(Ok(ExitStatus::Failure), |err| Err(err.into()));
                     };
 
                     debug!(
@@ -493,12 +488,10 @@ pub(crate) async fn install(
                         spec,
                         &interpreter,
                         settings.as_ref().into(),
+                        &network_settings,
                         &state,
                         Box::new(DefaultResolveLogger),
-                        connectivity,
                         concurrency,
-                        native_tls,
-                        allow_insecure_host,
                         &cache,
                         printer,
                         preview,
@@ -507,9 +500,11 @@ pub(crate) async fn install(
                     {
                         Ok(resolution) => (resolution, interpreter),
                         Err(ProjectError::Operation(err)) => {
-                            return diagnostics::OperationDiagnostic::native_tls(native_tls)
-                                .report(err)
-                                .map_or(Ok(ExitStatus::Failure), |err| Err(err.into()));
+                            return diagnostics::OperationDiagnostic::native_tls(
+                                network_settings.native_tls,
+                            )
+                            .report(err)
+                            .map_or(Ok(ExitStatus::Failure), |err| Err(err.into()));
                         }
                         Err(err) => return Err(err.into()),
                     }
@@ -532,13 +527,11 @@ pub(crate) async fn install(
             &resolution.into(),
             Modifications::Exact,
             settings.as_ref().into(),
+            &network_settings,
             &state,
             Box::new(DefaultInstallLogger),
             installer_metadata,
-            connectivity,
             concurrency,
-            native_tls,
-            allow_insecure_host,
             &cache,
             printer,
             preview,
@@ -551,7 +544,7 @@ pub(crate) async fn install(
         }) {
             Ok(environment) => environment,
             Err(ProjectError::Operation(err)) => {
-                return diagnostics::OperationDiagnostic::native_tls(native_tls)
+                return diagnostics::OperationDiagnostic::native_tls(network_settings.native_tls)
                     .report(err)
                     .map_or(Ok(ExitStatus::Failure), |err| Err(err.into()))
             }
