@@ -1,6 +1,7 @@
 use crate::common::{uv_snapshot, venv_bin_path, TestContext};
 use anyhow::Result;
 use assert_cmd::assert::OutputAssertExt;
+use assert_fs::fixture::{FileWriteStr, PathChild};
 use flate2::bufread::GzDecoder;
 use fs_err::File;
 use indoc::indoc;
@@ -278,3 +279,92 @@ fn preserve_executable_bit() -> Result<()> {
 
     Ok(())
 }
+
+/// Test `tool.uv.build-backend.module-name`.
+///
+/// We include only the module specified by `module-name`, ignoring the project name and all other
+/// potential modules.
+#[test]
+fn rename_module() -> Result<()> {
+    let context = TestContext::new("3.12");
+    let temp_dir = TempDir::new()?;
+
+    context
+        .temp_dir
+        .child("pyproject.toml")
+        .write_str(indoc! {r#"
+        [project]
+        name = "foo"
+        version = "1.0.0"
+
+        [tool.uv.build-backend]
+        module-name = "bar"
+
+        [build-system]
+        requires = ["uv_build>=0.5,<0.7"]
+        build-backend = "uv_build"
+    "#})?;
+
+    // This is the module we would usually include, but due to the renaming by `module-name` must
+    // ignore.
+    context
+        .temp_dir
+        .child("src/foo/__init__.py")
+        .write_str(r#"print("Hi from foo")"#)?;
+    // This module would be ignored from just `project.name`, but is selected due to the renaming.
+    context
+        .temp_dir
+        .child("src/bar/__init__.py")
+        .write_str(r#"print("Hi from bar")"#)?;
+
+    uv_snapshot!(context
+        .build_backend()
+        .arg("build-wheel")
+        .arg(temp_dir.path())
+        .env("UV_PREVIEW", "1"), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    foo-1.0.0-py3-none-any.whl
+
+    ----- stderr -----
+    "###);
+
+    context
+        .pip_install()
+        .arg(temp_dir.path().join("foo-1.0.0-py3-none-any.whl"))
+        .assert()
+        .success();
+
+    // Importing the module with the `module-name` name succeeds.
+    uv_snapshot!(Command::new(context.interpreter())
+        .arg("-c")
+        .arg("import bar")
+        // Python on windows
+        .env(EnvVars::PYTHONUTF8, "1"), @r###"
+    success: true
+    exit_code: 0
+    ----- stdout -----
+    Hi from bar
+
+    ----- stderr -----
+    "###);
+
+    // Importing the package name fails, it was overridden by `module-name`.
+    uv_snapshot!(Command::new(context.interpreter())
+        .arg("-c")
+        .arg("import foo")
+        // Python on windows
+        .env(EnvVars::PYTHONUTF8, "1"), @r###"
+    success: false
+    exit_code: 1
+    ----- stdout -----
+
+    ----- stderr -----
+    Traceback (most recent call last):
+      File "<string>", line 1, in <module>
+    ModuleNotFoundError: No module named 'foo'
+    "###);
+
+    Ok(())
+}
